SET @OLD_UNIQUE_CHECKS=@@UNIQUE_CHECKS, UNIQUE_CHECKS=0;
SET @OLD_FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0;
SET @OLD_SQL_MODE=@@SQL_MODE, SQL_MODE='TRADITIONAL';

CREATE SCHEMA IF NOT EXISTS `mydb` DEFAULT CHARACTER SET latin1 COLLATE latin1_swedish_ci ;
USE `mydb`;

-- -----------------------------------------------------
-- Table `mydb`.`users`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `mydb`.`users` ;

CREATE  TABLE IF NOT EXISTS `mydb`.`users` (
  `id` INT NOT NULL AUTO_INCREMENT ,
  `username` VARCHAR(45) NOT NULL ,
  `password` VARCHAR(45) NOT NULL ,
  `firstName` VARCHAR(45) NOT NULL ,
  `lastName` VARCHAR(45) NOT NULL ,
  `email` VARCHAR(45) NOT NULL ,
  PRIMARY KEY (`id`) )
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Table `mydb`.`roles`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `mydb`.`roles` ;

CREATE  TABLE IF NOT EXISTS `mydb`.`roles` (
  `id` INT NOT NULL AUTO_INCREMENT ,
  `name` VARCHAR(45) NOT NULL ,
  `description` VARCHAR(45) NOT NULL ,
  PRIMARY KEY (`id`) )
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Table `mydb`.`permissions`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `mydb`.`permissions` ;

CREATE  TABLE IF NOT EXISTS `mydb`.`permissions` (
  `id` INT NOT NULL AUTO_INCREMENT ,
  `name` VARCHAR(45) NOT NULL ,
  `description` VARCHAR(45) NULL ,
  `controller` VARCHAR(45) NOT NULL ,
  `action` VARCHAR(45) NOT NULL ,
  `bizrule` VARCHAR(255) NOT NULL DEFAULT '' ,
  PRIMARY KEY (`id`) ,
  UNIQUE INDEX `unique_access_rule` (`controller` ASC, `action` ASC, `bizrule` ASC) )
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Table `mydb`.`users_has_roles`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `mydb`.`users_has_roles` ;

CREATE  TABLE IF NOT EXISTS `mydb`.`users_has_roles` (
  `users_id` INT NOT NULL ,
  `roles_id` INT NOT NULL ,
  PRIMARY KEY (`users_id`, `roles_id`) ,
  INDEX `fk_users_has_roles_users` (`users_id` ASC) ,
  INDEX `fk_users_has_roles_roles` (`roles_id` ASC) ,
  CONSTRAINT `fk_users_has_roles_users`
    FOREIGN KEY (`users_id` )
    REFERENCES `mydb`.`users` (`id` )
    ON DELETE CASCADE
    ON UPDATE CASCADE,
  CONSTRAINT `fk_users_has_roles_roles`
    FOREIGN KEY (`roles_id` )
    REFERENCES `mydb`.`roles` (`id` )
    ON DELETE CASCADE
    ON UPDATE CASCADE)
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Table `mydb`.`permissions_has_roles`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `mydb`.`roles_has_permissions` ;

CREATE  TABLE IF NOT EXISTS `mydb`.`roles_has_permissions` (
  `permissions_id` INT NOT NULL ,
  `roles_id` INT NOT NULL ,
  PRIMARY KEY (`permissions_id`, `roles_id`) ,
  INDEX `fk_permissions_has_roles_permissions` (`permissions_id` ASC) ,
  INDEX `fk_permissions_has_roles_roles` (`roles_id` ASC) ,
  CONSTRAINT `fk_permissions_has_roles_permissions`
    FOREIGN KEY (`permissions_id` )
    REFERENCES `mydb`.`permissions` (`id` )
    ON DELETE CASCADE
    ON UPDATE CASCADE,
  CONSTRAINT `fk_permissions_has_roles_roles`
    FOREIGN KEY (`roles_id` )
    REFERENCES `mydb`.`roles` (`id` )
    ON DELETE CASCADE
    ON UPDATE CASCADE)
ENGINE = InnoDB;



SET SQL_MODE=@OLD_SQL_MODE;
SET FOREIGN_KEY_CHECKS=@OLD_FOREIGN_KEY_CHECKS;
SET UNIQUE_CHECKS=@OLD_UNIQUE_CHECKS;
